<?php
/**
 * @package Experience API for GamiPress by Grassblade
 * @version 1.6
 */
/*
Plugin Name: Experience API for GamiPress by Grassblade
Plugin URI: https://www.nextsoftwaresolutions.com/experience-api-for-gamipress/ 
Description: This plugin enables Experience API (xAPI) tracking of Gamification data generated using GamiPress plugin by integrating with GrassBlade xAPI Companion. 
Text Domain: grassblade-xapi-gamipress
Author: Next Software Solutions
Version: 1.6
Author URI: https://www.nextsoftwaresolutions.com
*/

class grassblade_gamipress {

	public $version = "1.6";

	function __construct() {
		if(!class_exists('grassblade_addons'))
		require_once(dirname(__FILE__)."/addon_plugins/functions.php");

		add_action('plugins_loaded', array($this, "textdomain"));
		add_action( 'admin_menu', array($this,'menu'), 10);

		include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
		add_action( 'plugins_loaded', array($this, "plugins_loaded") );
	}
	function textdomain() {
		$plugin_dir = basename(dirname(__FILE__));
		load_plugin_textdomain( 'grassblade-xapi-gamipress', $plugin_dir."/languages", $plugin_dir."/languages" );
	}
	function plugins_loaded() {
		if (is_plugin_active('gamipress/gamipress.php') && is_plugin_active('grassblade/grassblade.php')) {
			$enabled = get_option("grassblade_gamipress", true);
			if(!empty($enabled))
			$this->run();
		}
		else
			add_action( 'admin_notices', array($this, 'installation_notice') );
	}

	function menu() {
		global $submenu, $admin_page_hooks;
		$icon = plugin_dir_url(__FILE__)."img/icon-gb.png";

		if(empty( $admin_page_hooks[ "grassblade-lrs-settings" ] )) {
			add_menu_page("GrassBlade", "GrassBlade", "manage_options", "grassblade-lrs-settings", array($this, 'menu_page'), $icon, null);
		}

		add_submenu_page("grassblade-lrs-settings", "GrassBlade GamiPress", "GrassBlade GamiPress",'manage_options','grassblade-gamipress', array($this, 'menu_page'));
	}

	function menu_page() {

		if(!current_user_can("manage_options"))
			return;

		$enabled = get_option("grassblade_gamipress");

		if( !empty($_POST["submit"]) && check_admin_referer('grassblade_gamipress') ) {
			$enabled = intVal(isset($_POST["grassblade_gamipress"]));
			update_option("grassblade_gamipress", $enabled);
		}

		if($enabled === false) {
			$enabled = 1;
			update_option("grassblade_gamipress", $enabled);
		}

		?>
		<style type="text/css">
			div#grassblade_gamipress {
				padding: 30px;
				background: white;
				margin: 50px;
				border-radius: 5px;
			}
			div#grassblade_gamipress input[type=checkbox] {
			    margin-left: 50px;
			}
		</style>
		<style type="text/css">
			div#grassblade_gamipress {
				padding: 30px;
				background: white;
				margin: 50px;
				border-radius: 5px;
			}
			div#grassblade_gamipress input[type=checkbox] {
			    margin-left: 50px;
			}
		</style>
		<div id="grassblade_gamipress" class="wrap">
			<h3>GrassBlade GamiPress</h3>
			<form action="<?php echo $_SERVER['REQUEST_URI']; ?>" method="post">
				<?php wp_nonce_field( 'grassblade_gamipress' ); ?>
				<p style="padding: 20px;"><b><?php _e("Enable"); ?></b> <input name="grassblade_gamipress" type="checkbox" value="1" <?php if($enabled) echo 'checked="checked"'; ?>> </p>

				<p class="submit"><input type="submit" name="submit" id="submit" class="button button-primary" value="<?php _e("Save Changes"); ?>"></p>

			</form>
		</div>
		<?php 
	}

	function run(){

		add_filter( 'gamipress_activity_triggers', array($this,'xapi_activity_triggers') );

		add_filter( 'gamipress_specific_activity_triggers',  array($this, 'xapi_specific_activity_triggers') );

		add_filter( 'gamipress_activity_trigger_label', array($this, 'xapi_activity_trigger_label'), 10, 3 );

		add_filter( 'gamipress_specific_activity_trigger_label', array($this, 'xapi_specific_activity_trigger_label') );

		add_filter( 'gamipress_trigger_get_user_id', array($this, 'xapi_trigger_get_user_id'), 10, 3 );

		add_filter( 'gamipress_specific_trigger_get_id', array($this, 'xapi_specific_trigger_get_id'), 10, 3 );

		add_filter( 'gamipress_log_event_trigger_meta_data', array($this, 'xapi_log_event_trigger_meta_data'), 10, 5 );

		add_action( 'grassblade_completed', array($this, 'gamipress_xapi_content_completion'), 10, 3);  

		add_filter( 'gamipress_requirement_object', array($this, 'xapi_requirement_object'), 10, 2 );  

		add_action( 'gamipress_requirement_ui_html_after_achievement_post', array($this, 'xapi_requirement_ui_fields'), 10, 2 ); 

		add_action( 'gamipress_ajax_update_requirement', array($this, 'xapi_ajax_update_requirement'), 10, 2 );

		add_filter( 'user_has_access_to_achievement', array($this,'xapi_user_has_access_to_achievement'), 10, 6 );

		add_action( 'admin_enqueue_scripts', array($this,'custom_script_gamipress') );

		add_action( 'gamipress_insert_log', array($this,'send_earn_point_statements'), 10, 4 );

		include_once( dirname(__FILE__).'/reports_achievement_report/functions.php' );

	}

	function installation_notice() {
		?>
		<div class="error"><p>There are problems with <b>Experience API for GamiPress</b> plugin dependencies. Please install/enable <a href="https://www.nextsoftwaresolutions.com/grassblade-xapi-companion/">Grassblade xAPI Companion</a> and <a href="https://wordpress.org/plugins/gamipress/">GamiPress</a>.</p></div>
		<?php
	}

	/**
	 * Register Grassblade xAPI activity Triggers specific triggers
	 *
	 * @param array $triggers
	 * @return mixed
	 */
	function xapi_activity_triggers( $triggers ) {


	    $triggers['GrassBlade xAPI Companion'] = array(

	    	// Passed and Completed
	        'gamipress_grassblade_content_completion' => __( 'Completed any xAPI Content', 'grassblade-xapi-gamipress' ),
	        'gamipress_grassblade_specific_content_completion' => __( 'Completed a specific xAPI Content', 'grassblade-xapi-gamipress' ),

	        // Failed
	        'gamipress_grassblade_content_failed' => __( 'Failed any xAPI Content', 'grassblade-xapi-gamipress' ),
	        'gamipress_grassblade_specific_content_failed' => __( 'Failed a specific xAPI Content', 'grassblade-xapi-gamipress' ),

	        // Minimum Percentage
	        'gamipress_grassblade_content_with_min_per' => __( 'Completed xAPI Content with a minimum percentage', 'grassblade-xapi-gamipress' ),
	        'gamipress_grassblade_specific_content_with_min_per' => __( 'Completed a specific xAPI Content with a minimum percentage', 'grassblade-xapi-gamipress' ),

	        // Maximum Percentage
	        'gamipress_grassblade_content_with_max_per' => __( 'Completed xAPI Content with a maximum percentage', 'grassblade-xapi-gamipress' ),
	        'gamipress_grassblade_specific_content_with_max_per' => __( 'Completed a specific xAPI Content with a maximum percentage', 'grassblade-xapi-gamipress' ),

	        // Between Percentage
	        'gamipress_grassblade_content_between_per' => __( 'Completed xAPI Content scoring within a percentage range', 'grassblade-xapi-gamipress' ),
	        'gamipress_grassblade_specific_content_between_per' => __( 'Completed a specific xAPI Content scoring within a percentage range', 'grassblade-xapi-gamipress' ),
	        
	    );

	    return $triggers;
	}

	/**
	 * Register Grassblade xAPI specific activity triggers
	 *
	 *
	 * @param  array $specific_activity_triggers
	 * @return array
	 */
	function xapi_specific_activity_triggers( $specific_activity_triggers ) {

	    $specific_activity_triggers['gamipress_grassblade_specific_content_completion'] = array( 'gb_xapi_content' );
	    $specific_activity_triggers['gamipress_grassblade_specific_content_failed'] = array( 'gb_xapi_content' );
	    $specific_activity_triggers['gamipress_grassblade_specific_content_with_min_per'] = array( 'gb_xapi_content' );
	    $specific_activity_triggers['gamipress_grassblade_specific_content_with_max_per'] = array( 'gb_xapi_content' );
	    $specific_activity_triggers['gamipress_grassblade_specific_content_between_per'] = array( 'gb_xapi_content' );

	    return $specific_activity_triggers;
	}
	
	/**
	 * Build custom xAPI activity trigger label
	 *
	 * @param string    $title
	 * @param integer   $requirement_id
	 * @param array     $requirement
	 *
	 * @return string
	 */
	function xapi_activity_trigger_label( $title, $requirement_id, $requirement ) {

	    $percentage = ( isset( $requirement['gb_percentage'] ) ) ? absint( $requirement['gb_percentage'] ) : 0;
	    $min_per = ( isset( $requirement['gb_min_percentage'] ) ) ? absint( $requirement['gb_min_percentage'] ) : 0;
	    $max_per = ( isset( $requirement['gb_max_percentage'] ) ) ? absint( $requirement['gb_max_percentage'] ) : 0;

	    switch( $requirement['trigger_type'] ) {

	        // Minimum grade events
	        case 'gamipress_grassblade_content_with_min_per':
	            return sprintf( __( 'Completed a xAPI Content with a percentage of %d or higher', 'grassblade-xapi-gamipress' ), $percentage );
	            break;
	        case 'gamipress_grassblade_specific_content_with_min_per':
	            $achievement_post_id = absint( $requirement['achievement_post'] );
	            return sprintf( __( 'Complete %s - xAPI Content with a percentage of %d or higher', 'grassblade-xapi-gamipress' ), get_the_title( $achievement_post_id ), $percentage );
	            break;

	        // Maximum grade events
	        case 'gamipress_grassblade_content_with_max_per':
	            return sprintf( __( 'Completed a xAPI Content with a maximum percentage of %d', 'grassblade-xapi-gamipress' ), $percentage );
	            break;
	        case 'gamipress_grassblade_specific_content_with_max_per':
	         	$achievement_post_id = absint( $requirement['achievement_post'] );
	            return sprintf( __( 'Completed %s - xAPI Content with a maximum percentage of %d', 'grassblade-xapi-gamipress' ), get_the_title( $achievement_post_id ), $percentage );
	            break;

	        // Between grade events
	        case 'gamipress_grassblade_content_between_per':
	            return sprintf( __( 'Complete the xAPI Content with a percentage between %d and %d', 'grassblade-xapi-gamipress' ), $min_per, $max_per );
	            break;
	        case 'gamipress_grassblade_specific_content_between_per':
	            $achievement_post_id = absint( $requirement['achievement_post'] );
	            return sprintf( __( 'Complete %s - xAPI Content with a percentage between %d and %d', 'grassblade-xapi-gamipress' ), get_the_title( $achievement_post_id ), $min_per, $max_per );
	            break;

	    }

	    return $title;
	}

	/**
	 * Register LearnDash specific activity triggers labels
	 *
	 *
	 * @param  array $specific_activity_trigger_labels
	 * @return array
	 */
	function xapi_specific_activity_trigger_label( $specific_activity_trigger_labels ) {

	    $specific_activity_trigger_labels['gamipress_grassblade_specific_content_completion'] = __( 'Complete the xAPI Content %s', 'grassblade-xapi-gamipress' );
	    $specific_activity_trigger_labels['gamipress_grassblade_specific_content_failed'] = __( 'Failed the xAPI Content %s', 'grassblade-xapi-gamipress' );

	    return $specific_activity_trigger_labels;
	}

	/**
	 * Get user for a given trigger action.
	 *
	 *
	 * @param  integer $user_id user ID to override.
	 * @param  string  $trigger Trigger name.
	 * @param  array   $args    Passed trigger args.
	 * @return integer          User ID.
	 */
	function xapi_trigger_get_user_id( $user_id, $trigger, $args ) {

	    switch ( $trigger ) {
	        case 'gamipress_grassblade_content_completion':
	        case 'gamipress_grassblade_specific_content_completion':
	        case 'gamipress_grassblade_content_failed':
	        case 'gamipress_grassblade_specific_content_failed':
	        case 'gamipress_grassblade_content_with_min_per':
	        case 'gamipress_grassblade_specific_content_with_min_per':
	        case 'gamipress_grassblade_content_with_max_per':
	        case 'gamipress_grassblade_specific_content_with_max_per':
	        case 'gamipress_grassblade_content_between_per':
	        case 'gamipress_grassblade_specific_content_between_per':
	            $user_id = $args[1];
	            break;
	    }

	    return $user_id;
	}
	

	/**
	 * Get the id for a given specific trigger action.
	 *
	 * @param  integer  $specific_id Specific ID.
	 * @param  string  $trigger Trigger name.
	 * @param  array   $args    Passed trigger args.
	 *
	 * @return integer          Specific ID.
	 */
	function xapi_specific_trigger_get_id( $specific_id, $trigger = '', $args = array() ) {

	    switch ( $trigger ) {
	        case 'gamipress_grassblade_content_completion':
	        case 'gamipress_grassblade_specific_content_completion':
	        case 'gamipress_grassblade_content_failed':
	        case 'gamipress_grassblade_specific_content_failed':
	        case 'gamipress_grassblade_content_with_min_per':
	        case 'gamipress_grassblade_specific_content_with_min_per':
	        case 'gamipress_grassblade_content_with_max_per':
	        case 'gamipress_grassblade_specific_content_with_max_per':
	        case 'gamipress_grassblade_content_between_per':
	        case 'gamipress_grassblade_specific_content_between_per':
	            $specific_id = $args[0];
	            break;
	    }

	    return $specific_id;
	}
	

	/**
	 * Extended meta data for event trigger logging
	 *
	 * @param array 	$log_meta
	 * @param integer 	$user_id
	 * @param string 	$trigger
	 * @param integer 	$site_id
	 * @param array 	$args
	 *
	 * @return array
	 */
	function xapi_log_event_trigger_meta_data( $log_meta, $user_id, $trigger, $site_id, $args ) {

	    switch ( $trigger ) {
	    	case 'gamipress_grassblade_content_completion':
	        case 'gamipress_grassblade_specific_content_completion':
	        case 'gamipress_grassblade_content_failed':
	        case 'gamipress_grassblade_specific_content_failed':
	        	$log_meta['xapi_content_id'] = $args[0];
	            break;
	        case 'gamipress_grassblade_content_with_min_per':
	        case 'gamipress_grassblade_specific_content_with_min_per':
	        case 'gamipress_grassblade_content_with_max_per':
	        case 'gamipress_grassblade_specific_content_with_max_per':
	        case 'gamipress_grassblade_content_between_per':
	        case 'gamipress_grassblade_specific_content_between_per':
	        	$log_meta['xapi_content_id'] = $args[0];
	            $log_meta['percentage'] = $args[2];
	            break;
	    }

	    return $log_meta;
	}

	/**
	 * Completion xAPI Content Statement.
	 *
	 * @param obj $statement.
	 * @param int|string $xapi_content_id xAPI Content ID.
	 * @param obj $user User Object.
	 */
	function gamipress_xapi_content_completion($statement, $xapi_content_id, $user) {

		grassblade_show_trigger_debug_messages( "gamipress_xapi_content_completion started");

		$statement = json_decode($statement);

		$result = @$statement->result;

	    $percentage = isset($statement->result->score->scaled)? $statement->result->score->scaled*100:((!empty($statement->result->score->max) && isset($statement->result->score->raw))? $statement->result->score->raw*100/($statement->result->score->max - @$statement->result->score->min):100);
	    $percentage = round($percentage, 2);

	    $xapi_content = get_post_meta($xapi_content_id, "xapi_content", true);
	    if(isset($xapi_content["passing_percentage"]) && trim($xapi_content["passing_percentage"]) == "") {
	        if(isset($statement->result->success)) {
	            $status = (empty($statement->result->success) || is_string($statement->result->success) && $statement->result->success == "false")? "Failed":"Passed";
	        }
	        else
	            $status = "Completed";
	    }
	    else
	    {
	        $pass = ($percentage >= @$xapi_content["passing_percentage"])? 1:0;
	        $status = !empty($pass)? "Passed":"Failed";
	    }

	    if ($status == 'Failed') {
	    	// Failed any xAPI Content
	    	do_action( 'gamipress_grassblade_content_failed', $xapi_content_id, $user->ID, $percentage);

	    	do_action( 'gamipress_grassblade_specific_content_failed', $xapi_content_id, $user->ID, $percentage);
	    } 

	     if ($status == 'Passed' || $status == 'Completed') {
	    	// Completed or Passed xAPI Content
    		do_action( 'gamipress_grassblade_content_completion', $xapi_content_id, $user->ID, $percentage);

    		do_action( 'gamipress_grassblade_specific_content_completion', $xapi_content_id, $user->ID, $percentage);
	    } 

    	do_action( 'gamipress_grassblade_content_with_min_per', $xapi_content_id, $user->ID, $percentage);

    	do_action( 'gamipress_grassblade_specific_content_with_min_per', $xapi_content_id, $user->ID, $percentage);

    	do_action( 'gamipress_grassblade_content_with_max_per', $xapi_content_id, $user->ID, $percentage);

    	do_action( 'gamipress_grassblade_specific_content_with_max_per', $xapi_content_id, $user->ID, $percentage);

    	do_action( 'gamipress_grassblade_content_between_per', $xapi_content_id, $user->ID, $percentage);

    	do_action( 'gamipress_grassblade_specific_content_between_per', $xapi_content_id, $user->ID, $percentage);
	}	

	/**
	 * Add the score field to the requirement object
	 *
	 * @param $requirement
	 * @param $requirement_id
	 *
	 * @return array
	 */
	function xapi_requirement_object( $requirement, $requirement_id ) {

	    if( isset( $requirement['trigger_type'] )
	        && ( $requirement['trigger_type'] === 'gamipress_grassblade_content_with_min_per'
	            || $requirement['trigger_type'] === 'gamipress_grassblade_specific_content_with_min_per'
	            || $requirement['trigger_type'] === 'gamipress_grassblade_content_with_max_per'
	            || $requirement['trigger_type'] === 'gamipress_grassblade_specific_content_with_max_per' ) ) {

	        // Minimum/Maximum grade percent
	        $requirement['gb_percentage'] = get_post_meta( $requirement_id, '_gamipress_gb_percentage', true );

	    }

	    if( isset( $requirement['trigger_type'] )
	        && ( $requirement['trigger_type'] === 'gamipress_grassblade_content_between_per'
	            || $requirement['trigger_type'] === 'gamipress_grassblade_specific_content_between_per' ) ) {

	        // Between grade percent
	        $requirement['gb_min_percentage'] = get_post_meta( $requirement_id, '_gamipress_gb_min_percentage', true );
	        $requirement['gb_max_percentage'] = get_post_meta( $requirement_id, '_gamipress_gb_max_percentage', true );

	    }

	    return $requirement;
	}
	

	/**
	 * Category field on requirements UI
	 *
	 * @param $requirement_id
	 * @param $post_id
	 */
	function xapi_requirement_ui_fields( $requirement_id, $post_id ) {

	    $percentage = absint( get_post_meta( $requirement_id, '_gamipress_gb_percentage', true ) );
	    $min_per = get_post_meta( $requirement_id, '_gamipress_gb_min_percentage', true );
	    $max_per = get_post_meta( $requirement_id, '_gamipress_gb_max_percentage', true );
	    ?>

	    <span class="gb-percentage"><input type="text" value="<?php echo $percentage; ?>" size="3" maxlength="3" placeholder="100" />%</span>
	    <span class="gb-min-percentage"><input type="text" value="<?php echo ( ! empty( $min_per ) ? absint( $min_per ) : '' ); ?>" size="3" maxlength="3" placeholder="Min" />% -</span>
	    <span class="gb-max-percentage"><input type="text" value="<?php echo ( ! empty( $max_per ) ? absint( $max_per ) : '' ); ?>" size="3" maxlength="3" placeholder="Max" />%</span>

	    <?php
	}
	

	/**
	 * Custom handler to save the score on requirements UI
	 *
	 * @param $requirement_id
	 * @param $requirement
	 */
	function xapi_ajax_update_requirement( $requirement_id, $requirement ) {

		if( isset( $requirement['trigger_type'] )
	        && ( $requirement['trigger_type'] === 'gamipress_grassblade_content_with_min_per'
	            || $requirement['trigger_type'] === 'gamipress_grassblade_specific_content_with_min_per'
	            || $requirement['trigger_type'] === 'gamipress_grassblade_content_with_max_per'
	            || $requirement['trigger_type'] === 'gamipress_grassblade_specific_content_with_max_per' ) ) {

	        // Save the percentage field
	        update_post_meta( $requirement_id, '_gamipress_gb_percentage', $requirement['gb_percentage'] );
	    }

	    if( isset( $requirement['trigger_type'] )
	        && ( $requirement['trigger_type'] === 'gamipress_grassblade_content_between_per'
	            || $requirement['trigger_type'] === 'gamipress_grassblade_specific_content_between_per' ) ) {

	        // Between grade percent
	        update_post_meta( $requirement_id, '_gamipress_gb_min_percentage', $requirement['gb_min_percentage'] );
	        update_post_meta( $requirement_id, '_gamipress_gb_max_percentage', $requirement['gb_max_percentage'] );
	    }
	}

	/**
	 * Checks if an user is allowed to work on a given requirement related to a minimum of score
	 *
	 * @since  1.0.0
	 *
	 * @param bool $return          The default return value
	 * @param int $user_id          The given user's ID
	 * @param int $requirement_id   The given requirement's post ID
	 * @param string $trigger       The trigger triggered
	 * @param int $site_id          The site id
	 * @param array $args           Arguments of this trigger
	 *
	 * @return bool True if user has access to the requirement, false otherwise
	 */
	function xapi_user_has_access_to_achievement( $return = false, $user_id = 0, $requirement_id = 0, $trigger = '', $site_id = 0, $args = array() ) {

	    // If we're not working with a requirement, bail here
	    if ( ! in_array( get_post_type( $requirement_id ), gamipress_get_requirement_types_slugs() ) )
	        return $return;

	    // Check if user has access to the achievement ($return will be false if user has exceed the limit or achievement is not published yet)
	    if( ! $return )
	        return $return;

	    // If is minimum percentage trigger, rules engine needs to check the minimum percentage
	    if( $trigger === 'gamipress_grassblade_content_with_min_per'
	        || $trigger === 'gamipress_grassblade_specific_content_with_min_per' ) {

	        $percentage = absint( $args[2] );

	        $required_percentage = absint( get_post_meta( $requirement_id, '_gamipress_gb_percentage', true ) );

	        // True if there is percentage is bigger than required percentage
	        $return = (bool) ( $percentage >= $required_percentage );
	    }

	    // If is maximum percentage trigger, rules engine needs to check the maximum percentage
	    if( $trigger === 'gamipress_grassblade_content_with_max_per'
	        || $trigger === 'gamipress_grassblade_specific_content_with_max_per' ) {

	        $percentage = absint( $args[2] );

	        $required_percentage = absint( get_post_meta( $requirement_id, '_gamipress_gb_percentage', true ) );

	        // True if there is percentage is lower than required percentage
	        $return = (bool) ( $percentage <= $required_percentage );
	    }

	    // If is between score trigger, rules engine needs to check the minimum and maximum score allowed
	    if( $trigger === 'gamipress_grassblade_content_between_per'
	        || $trigger === 'gamipress_grassblade_specific_content_between_per' ) {

	        $percentage = absint( $args[2] );

	        $min_percentage = absint( get_post_meta( $requirement_id, '_gamipress_gb_min_percentage', true ) );
	        $max_percentage = absint( get_post_meta( $requirement_id, '_gamipress_gb_max_percentage', true ) );

	        // True if there is percentage is bigger than min percentage and lower than max percentage
	        $return = (bool) ( $percentage >= $min_percentage && $percentage <= $max_percentage );
	    }

	    // Send back our eligibility
	    return $return;
	}

	function custom_script_gamipress(){
	  wp_enqueue_script( 'gb-gamipress-script', plugin_dir_url( __FILE__ ) . 'js/gb-gamipress.js', array( 'jquery' ) , $this->version);
	} 

	/**
	 * Send Statements to LRS , When anybody earns points
	 *
	 * @param int 	    $log_id
	 * @param array 	$log_data
	 * @param array 	$log_meta
	 * @param int    	$user_id
	 *
	 */
	function send_earn_point_statements( $log_id, $log_data, $log_meta, $user_id ){

		if ($log_data['type'] == 'points_earn') {
			$grassblade_settings = grassblade_settings();

		    $grassblade_tincan_endpoint = $grassblade_settings["endpoint"];
		    $grassblade_tincan_user = $grassblade_settings["user"];
		    $grassblade_tincan_password = $grassblade_settings["password"];
			$grassblade_tincan_track_guest = $grassblade_settings["track_guest"];

			$user = get_userdata($user_id);

			$xapi = new NSS_XAPI($grassblade_tincan_endpoint, $grassblade_tincan_user, $grassblade_tincan_password);
			$actor = grassblade_getactor($grassblade_tincan_track_guest, "1.0", $user);

			if(empty($actor))
			{
				grassblade_debug("No Actor. Shutting Down.");
				return;
			}

			$achievement = get_post($log_meta['achievement_id']);
			$achievement_url = grassblade_post_activityid($log_meta['achievement_id']);
			$achievement_title = $achievement->post_title;

			$xapi->set_verb('earned');
			$xapi->set_actor_by_object($actor);
			$xapi->set_object($achievement_url, $achievement_title, $log_data['title'], '','Activity');
			$statement = $xapi->build_statement();
			
			$ret = $xapi->SendStatements(array($statement));
		}
	}

} // end of grassblade_gamipress class
	
$gbgp = new grassblade_gamipress();

